
function b = learn_beta(X,Y,lambda,standardize)
%lassoFit1(X,Y,weights,lambda,alpha,dfmax,standardize,reltol,lambdaMax,ever_active)
% ------------------------------------------------------
% Perform model fit for each lambda and the given alpha
% ------------------------------------------------------

[N,P] = size(X);
if standardize
    [X0,muX,sigmaX] = zscore(X,1);
else
     muX = mean(X,1);
     X0 = bsxfun(@minus,X,muX);
     sigmaX = 1;
end

muY = mean(Y);
Y0 = bsxfun(@minus,Y,muY);
totalweight = N;
% b will be the current coefficient estimate, iteratively updated.
% Because we retain b from one value of lambda to the next,
% we get a de facto warm start.
b = zeros(P,1);
reltol=0.00001;
% Preallocate the returned matrix of coefficients, B, and the intercepts.
ever_active =ones(1,P);   
active = ever_active;

% Iterative coordinate descent until converged
while true

    bold = b;

    [b,active] = cdescentCycle1(X0,Y0,b,active,lambda,totalweight);

    if max( abs(b-bold)./max(1.0,min(abs(b),abs(bold))) ) < reltol
        % Cycling over the active set converged.
        % Do one full pass through the predictors.
        % If there is no predictor added to the active set, we're done.
        % Otherwise, resume the coordinate descent iterations.
        bold = b;
        [b,new_active] = cdescentCycle1(X0,Y0,b,active,lambda,totalweight);

        if isequal(new_active, active)
            break
        elseif max( abs(b-bold)./max(1.0,min(abs(b),abs(bold))) ) < reltol
            % We didn't change the coefficients enough by the full pass 
            % through the predictors to warrant continuing the iterations.
            % The active coefficients after this pass and the
            % active coefficients prior to this pass differ.
            % This implies that a coefficient changed between zero
            % and a small numeric value.  There is no "fit-wise" reason
            % to prefer the before and after coefficients, so we
            % choose the more parsimonious alternative.
            if sum(new_active) > sum(active)
                b = bold;
            else
                active = new_active;
            end
            break
        else
            active = new_active;
        end
    end

end
end

function [b,active] = cdescentCycle1(X0, Y0, b, active, lambda,totalweight)

r = Y0 - X0*b;

for j=find(active);
    bjold = b(j);
    threshold2=lambda(j);
    % Regress j-th partial residuals on j-th predictor
    rj = r + b(j)*X0(:,j); % calcuate the jth partial residueal
    bj = X0(:,j)'*rj/totalweight ; % regress jth pr on jth predictor
    
    % Soft thresholding
    %b(j) = sign(bj) .* max(abs(bj) - threshold, 0);
    b(j) = sign(bj) .* max(abs(bj) - threshold2, 0);
    if b(j) == 0
        active(j) = false;
    end
    r = r - X0(:,j)*(b(j)-bjold); % in case of b(j)=0, we need cut the effect of b(j)
end %cdescentCycle

end

