function num_association = rDriver(y, x, mf, params)
%y: mRNA expression
%x: mutation matrix
%FIS: Functional impact score

if nargin< 4
   error('too few input arguments') 
end

if ~isfield(params,'mb0')
    params.mb0=0;
end
if ~isfield(params,'mb1')
    params.mb1=1;
end
if ~isfield(params,'e')
    params.e=3;
end
if ~isfield(params,'e2')
    params.e2=3;
end
if ~isfield(params,'TuningFactor')
    params.TuningFactor=0.00001;
end

[nmutations, msamples] = size(x); 
[ngenes, nsamples] = size(y);% genes X array expression
num_mfeats = size( mf{1}, 2 );%num of metafeature for each marker

if nsamples ~= msamples
    error('x and y dimentions not match');
end


mb0=params.mb0; mb1=params.mb1;e=params.e;e2=params.e2;
TuningFactor=params.TuningFactor;

m_beta = zeros(ngenes,num_mfeats);	
solution = zeros(ngenes,nmutations);
maxiters =10;
standarized = 1;
L = (y)*(x');
lambda_max = 2*max(abs(L)')'; 
for i=1:1:ngenes
    if mod(i,100) == 0
        fprintf('gene = %g/ %g\n', i, ngenes);
    end
mb =  1*ones(num_mfeats,1); 
mw = zeros(1, nmutations);
lambda =TuningFactor* lambda_max(i);
obj = 0;
    for iter = 1 : maxiters 
       [ma] = calculate_feature_relevance( mf, mb0, mb1, mb );
       mw = learn_beta(x',y(i,:)',ma*lambda,standarized);
       if( iter == 1 )
          params.mw0 = mw;	
       end
       if( maxiters>1 )
          if( iter==1 )
             mb = 1*ones(num_mfeats,1);
          end
          if all(mw==0)
              break;
          end          
          mb= learn_feature_weight(mf,mb,mw,mb0,mb1,e,e2); 
       end
       preobj = obj;
       obj = sum( sum( (y(i,:) - mw' * x).^2 ) ) + sum( sum(lambda* abs(mw) .* ma ) ) ;
       if abs(preobj-obj) < 1e-5
          break 
       end
      %fprintf('\niter = %g\tobj = %g\tdel = %g\n\n',iter, obj, preobj-obj);
    end
[ma] = calculate_feature_relevance( mf, mb0, mb1, mb );
solution(i,:) = mw;
m_beta(i,:) = mb';
end
solution_binary=zeros(size(solution));
solution_binary(solution~=0)=1;
num_association = sum(solution_binary,1);

end
